--[[
A GENERIC SPRITE CLASS
]]

require 'hardoncollider.polygon'
local class = require 'middleclass'
require 'constants'
Sprite = class('Sprite')

-- set the sprites internal variables
function Sprite:initialize(x, y, x_vel, y_vel)
	self.x = x
	self.y = y
	self.x_vel = x_vel
	self.y_vel = y_vel
end

-- updates the sprites position and velocities
function Sprite:move(dt)

	-- update position
	self.x = self.x + self.x_vel*dt
	self.y = self.y + self.y_vel*dt

	-- wraps the coordinates around the edge of the screen
	self.x, self.y = overside(self.x, self.y)

end

--[[
FUNCTION FOR SPRITES THAT IMPLEMENT THE HARDONCOLLIDER LIBRARY
Note: does NOT implement Sprite class.
]]
ColliderSprite = class('ColliderSprite')

function ColliderSprite:initialize(shape, x_vel, y_vel, lin_slow, rot_vel, rot_slow, color, border_color, image)
	self.img = image
	if self.img ~= nil then
		self.img_ox = self.img:getWidth()/2
		self.img_oy = self.img:getHeight()/3
	end
	self.shape = shape
	self.shape.parent = self
	self.color = copy(color)
	self.border_color = copy(border_color)
	self.rot_vel = rot_vel
	self.x_vel = x_vel
	self.y_vel = y_vel
	self.lin_slow = lin_slow
	self.rot_slow = rot_slow
	self.angle = self.shape:rotation()
	self.rot_vel = rot_vel
	--initial definition of HC objects is done via the Shape argument; thus, x and y values are derived
	--from this shape and updated every time the object moves
	self.x, self.y = shape:center()
end

function ColliderSprite:move(dt)
	--this calls the HardonCollider move function on the sprite object's Shape field
	self.shape:move(self.x_vel * dt, self.y_vel * dt)
	--and rotate function
	self.shape:rotate(self.rot_vel * dt)
	--this updates the x and y variables; strictly speaking we could
	--change everything that uses object.x/y to call object.shape:center() instead,
	--but this is for ease of use/laziness
	self.x, self.y = self.shape:center()
	--same deal with self.angle
	self.angle = self.shape:rotation()
	--velocity values exist wholly outside of the Collider realm, but they get
	--passed into the move function
	self.x_vel = self.x_vel*math.pow(self.lin_slow, dt)
	self.y_vel = self.y_vel*math.pow(self.lin_slow, dt)
	self.rot_vel = self.rot_vel*math.pow(self.rot_slow, dt)
	--overside_c moves the Shape object, as there's no way to manually set the Shape's x/y value
	self:overside_c()
end

--[[
note: it's likely overkill to use the ColliderSprite object for objects that don't need
to rotate or have their movement damped (ie most things that aren't the ship) but I was
too lazy to abstract out classes / it was a lot cleaner this way. if there are significant
performance benefits to having, say, IrrotationalColliderSprite or PhysicsImmuneColliderSprite
that might have to happen down the road.
]]

function ColliderSprite:turn(deltaRotVel)
	self.rot_vel = self.rot_vel + deltaRotVel
end

-- wraps coordinates with collider object
function ColliderSprite:overside_c()
	self.shape:moveTo(mod(self.x, constants.SCREEN_WIDTH), mod(self.y, constants.SCREEN_HEIGHT))
end

function ColliderSprite:draw()
	--uses HardonCollider:Shape built in draw function - very basic.
	--likely that every object that needs to check collisions will eventually
	--just be represented by a bitmap file.
	if self.img == nil then
		love.graphics.setColor(self.color)
		self.shape:draw('fill')
		love.graphics.setColor(self.border_color)
		self.shape:draw('line')
	else
		love.graphics.setColor(colors.WHITE)
		love.graphics.draw(self.img, self.x, self.y, self.angle, 1, 1, self.img_ox, self.img_oy)
	end
end
