--[[

UPGRADES FOR THE PLAYER

]]

local class = require 'middleclass'
require 'constants'
require 'sprites'
Upgrade = class('Upgrade', ColliderSprite)

function Upgrade:initialize(hc, color, sound_name)
	self.objtype = types.UPGRADE
	self.sound = sound_name
	x_vel, y_vel = polar_to_cartesian(constants.UPGRADE_SPEED,(math.random()*2*math.pi))
	x = 0
	y = 0
	-- spawns on left or right
	if (math.random() > 0.5) then
		if (math.random() > 0.5) then
			y = math.random(constants.SCREEN_HEIGHT)
		end
	-- spawns on top or bottom
	else
		if (math.random() > 0.5) then
			x = math.random(constants.SCREEN_WIDTH)
		end
	end
	self.diameter = constants.UPGRADE_DIAMETER
	local shape = hc:addCircle(x, y, self.diameter)
	self.lifetime = constants.UPGRADE_LIFETIME
	self.alive = true
	ColliderSprite.initialize(self, shape, x_vel, y_vel, 1.0, 0, 1.0, color, colors.NO_COLOR, nil)
	self.ps = self:load_particle_system()
end

function Upgrade:move(dt)
	ColliderSprite.move(self, dt)
	self.ps:setPosition(self.x, self.y)
	self.ps:update(dt)

	self.lifetime = self.lifetime - dt
	if self.lifetime < dt * constants.UPGRADE_EMISSION_RATE then
		--decrease the alpha of the upgrade until it is invisible
		if self.lifetime > 0 then
			self.ps:setEmissionRate(self.lifetime)
		else self:kill()
		end
	end

end

function Upgrade:kill()
	self.alive = false
end

function Upgrade:draw()
	love.graphics.draw(self.ps, 0, 0)
end

function Upgrade:load_particle_system()
	local ps = love.graphics.newParticleSystem(love.graphics.newImage('assets/images/fade_circle.png'), 1)
	ps:setBufferSize( 200 )
  ps:setParticleLifetime( 0.5 )
	ps:setSpeed( 0, 25 )
	ps:setSpread( math.rad(360) )
	ps:setEmissionRate( constants.UPGRADE_EMISSION_RATE )
	ps:setColors(unpack(all_values(self.color, self.border_color)))
	return ps
end

function Upgrade:collide(obj)

	if obj.objtype == types.SHIP then
		self:upgrade(obj)
		self:kill()
		return self.sound
	end

end


-- ============================================================= --

--[[
A HEALTH UPGRADE (+1 health)
]]

HealthUp = class('HealthUp', Upgrade)

function HealthUp:initialize(hc)
	Upgrade.initialize(self, hc, colors.HEALTH_UP, "healthup")
end


function HealthUp:upgrade(ship)
	ship.health = bound(ship.health+1, 0, ship.max_health)
end


-- ============================================================= --

--[[
A MAX HEALTH UPGRADE
]]

MaxHealth = class('MaxHealth', Upgrade)

function MaxHealth:initialize(hc)
	Upgrade.initialize(self, hc, colors.MAX_HEALTH, "maxhealth")
end


function MaxHealth:upgrade(ship)
	ship.health = ship.max_health
end


-- ============================================================= --

--[[
AN UPGRADE TO INCREASE A SHIP'S MAX HEALTH
]]

MaxHealthUp = class('MaxHealthUp', Upgrade)

function MaxHealthUp:initialize(hc)
	Upgrade.initialize(self, hc, colors.MAX_HEALTH_UP, "maxhealthup")
end


function MaxHealthUp:upgrade(ship)
	ship.max_health = ship.max_health+2
	ship.health = ship.max_health
end

function MaxHealthUp:move(dt)
	self.ps:setColors(unpack(all_values(self.color,{math.random(255),math.random(255),math.random(255),255})))
	Upgrade.move(self, dt)
end

-- ============================================================= --

--[[
AN UPGRADE TO GIVE THE SHIP A TEMPORARY SHIELD
]]

ShieldUp = class('ShieldUp', Upgrade)

function ShieldUp:initialize(hc)
	Upgrade.initialize(self, hc, colors.MAX_HEALTH_UP, "powerup")
end


function ShieldUp:upgrade(ship)
	ship.shield.on = true
end

-- ============================================================= --


--[[
AN UPGRADE TO GIVE THE SHIP EXTRA GOLD
]]

Gold = class('Gold', Upgrade)

function Gold:initialize(hc)
	Upgrade.initialize(self, hc, colors.GOLD, "money")
end


function Gold:upgrade(ship)
	--lol
end



-- ============================================================= --

-- a function to return a random upgrade
function get_upgrade()
	rand = math.random()
	if rand < constants.UPGRADE_CUTOFFS[1] then
		return Gold(hc)
	elseif rand < constants.UPGRADE_CUTOFFS[2] then
		return HealthUp(hc)
	elseif rand < constants.UPGRADE_CUTOFFS[3] then
		return MaxHealth(hc)
	elseif rand < constants.UPGRADE_CUTOFFS[4] then
		return MaxHealthUp(hc)
	end
end
